local mod = UpgradeList:NewModule("Import")
local L = LibStub("AceLocale-3.0"):GetLocale("UpgradeList")

local UPGRADELIST_VERSION = GetAddOnMetadata('UpgradeList', 'Version')

local RawrInputExplanation = "To import items from Rawr:\n\n  1) Open Rawr and load the character you intend to import items to\n  2) Go to file, Export to Rawr addon, and copy all the code\n  3) Paste the code below (Can lag or disconnect you if there are too many items)\n  4) Select which spec the items are for and click the 'Import' button at the bottom"

function UpgradeList:RawrImport()
	InterfaceOptionsFrame:Hide()
	GameMenuFrame:Hide()
	if self.RawrImportFrame then
		self.RawrImportFrame:Show()
	else
		local frame = CreateFrame("frame", "RawrFrame", UIParent)
		frame:SetFrameStrata("DIALOG")
		frame:SetWidth(600)
		frame:SetHeight(700)
		frame:SetScale(1)
		
		frame:SetBackdrop({bgFile = "Interface/DialogFrame/UI-DialogBox-Background-Dark", 
			edgeFile = "Interface/DialogFrame/UI-DialogBox-Border",
			tile = true, tileSize = 32, edgeSize = 32, 
			insets = { left = 11, right = 11, top = 10, bottom = 10 }})

		frame:SetBackdropColor(0,0,0,0.8)

		frame:SetPoint("TOP",0,-80)
		
		frame:SetMovable(true)
		frame:EnableMouse(true)
		frame:RegisterForDrag("LeftButton")
		frame:SetScript("OnDragStart", frame.StartMoving)
		frame:SetScript("OnDragStop", frame.StopMovingOrSizing)
		
		tinsert(UISpecialFrames,"RawrFrame")

		frame.title = frame:CreateTexture(nil,"ARTWORK")
		frame.title:SetTexture("Interface/DialogFrame/UI-DialogBox-Header")
		frame.title:SetWidth(280)
		frame.title:SetHeight(64)
		frame.title:SetPoint("TOP", frame, 0, 12)
		frame.texture = frame.title

		frame.title.text = frame:CreateFontString(nil, "ARTWORK", "GameFontNormalSmall")
		frame.title.text:SetText('Rawr Import')
		frame.title.text:SetPoint("TOP", frame, 0, -3)
		
		frame.closebutton = CreateFrame('Button', "CloseButton", frame, "UIPanelCloseButton")
		frame.closebutton:SetPoint("TOPRIGHT", -1, -2)
		frame.closebutton:SetWidth(40)
		frame.closebutton:SetHeight(40)
		
		--Scroll Frame
		frame.scrollframe = CreateFrame("ScrollFrame","CodeWindow",frame)
		frame.scrollframe:SetPoint("TOPLEFT", frame, "TOPLEFT", 10, -120)
		frame.scrollframe:SetPoint("BOTTOMRIGHT", frame, "BOTTOMRIGHT", -35, 84)
		frame.scrollframe:SetBackdrop({bgFile = "Interface/DialogFrame/UI-DialogBox-Background-Dark", 
			edgeFile = "Interface/GLUES/COMMON/TextPanel-Border",
			tile = true, tileSize = 32, edgeSize = 15, 
			insets = { left = 5, right = 5, top = 5, bottom = 5 }})
		frame.scrollframe:SetBackdropColor(0,0,0,0.8)
		
		--Slider
		frame.slider = CreateFrame("Slider","Scroll Bar",frame.scrollframe,"UIPanelScrollBarTemplate")
		frame.slider:SetPoint("TOPLEFT", frame.scrollframe, "TOPRIGHT", 5, -15)
		frame.slider:SetPoint("BOTTOMLEFT", frame.scrollframe, "BOTTOMRIGHT", 5, 15)
		frame.slider:SetPoint("LEFT", frame.scrollframe, "RIGHT", 0, 0)
		frame.slider.scrollStep = 1
		frame.slider:SetWidth(16)
		frame.slider:SetMinMaxValues(0, 495)
		frame.slider:SetOrientation("VERTICAL")
		frame.slider:SetValue(0)
		frame.slider:SetValueStep(1)
		frame.slider:SetScript("OnValueChanged", function(self,value)
			self:GetParent():SetVerticalScroll(value)
		end)
		frame.slider.bg = frame.slider:CreateTexture(nil,"BACKGROUND")
		frame.slider.bg:SetAllPoints(frame.slider)
		frame.slider.bg:SetTexture(0.2,0.2,0.2,0.8)
		
		frame.PrimarySpecRadioButton = CreateFrame('CheckButton', "ImportPrimarySpecRadioButton", frame, "UIRadioButtonTemplate")
		frame.PrimarySpecRadioButton:SetHeight(20)
		frame.PrimarySpecRadioButton:SetWidth(20)
		frame.PrimarySpecRadioButton:SetChecked(true)
		frame.PrimarySpecRadioButton:SetPoint("BOTTOM", frame, "BOTTOM", -125, 60)
		_G[frame.PrimarySpecRadioButton:GetName() .. "Text"]:SetText(L["Primary Spec"])
		frame.PrimarySpecRadioButton:SetScript("OnClick", function (self, button, down)
			if GetNumTalentGroups("player") == 1 then
				SecondarySpecRadioButton:Disable()
				frame.PrimarySpecRadioButton:SetChecked(true)
			else
				frame.SecondarySpecRadioButton:SetChecked(false)
			end
		frame.PrimarySpecRadioButton:SetChecked(true)
		end)
		
		frame.SecondarySpecRadioButton = CreateFrame('CheckButton', "ImportSecondarySpecRadioButton", frame, "UIRadioButtonTemplate")
		frame.SecondarySpecRadioButton:SetHeight(20)
		frame.SecondarySpecRadioButton:SetWidth(20)
		frame.SecondarySpecRadioButton:SetChecked(false)
		frame.SecondarySpecRadioButton:SetPoint("BOTTOM", frame, "BOTTOM", 50, 60)
		_G[frame.SecondarySpecRadioButton:GetName() .. "Text"]:SetText(L["Secondary Spec"])
		if GetNumTalentGroups("player") == 1 then
			_G[frame.SecondarySpecRadioButton:GetName() .. "Text"]:SetTextColor(0.5, 0.5, 0.5)
			frame.SecondarySpecRadioButton:Disable()
		end
		frame.SecondarySpecRadioButton:SetScript("OnClick", function (self, button, down)
			frame.SecondarySpecRadioButton:SetChecked(true)
			frame.PrimarySpecRadioButton:SetChecked(false)
		end)
		
		--EditBox
		frame.editbox = CreateFrame("EditBox","Editbox",frame.scrollframe)
		frame.editbox:SetPoint("TOPLEFT", 5, -5)
		frame.editbox:SetWidth(frame.scrollframe:GetWidth())
		frame.editbox:SetFontObject(ChatFontNormal)
		frame.editbox:SetMultiLine(true)
		frame.editbox:SetTextInsets(4,5,6,5)
		frame.editbox:SetScript("OnTextChanged", function(self)
			if frame.editbox:GetText() == "" then
				frame.button:Disable()
			else
				frame.button:Enable()
			end
			if frame.editbox:GetHeight() > 495 then
				frame.slider:SetMinMaxValues(0, frame.editbox:GetHeight())
			else
				frame.slider:SetMinMaxValues(0, 495)
			end
		end)
		frame.editbox:SetScript("OnHide", function(self)
			frame.editbox:SetText("")
			frame.PrimarySpecRadioButton:SetChecked(true)
			frame.SecondarySpecRadioButton:SetChecked(false)
		end)
		
		frame.scrollframe:SetScrollChild(frame.editbox)
		
		frame.heading = CreateFrame('Frame', "Explanation", frame)
		frame.heading:SetPoint("TOPLEFT", frame, "TOPLEFT", 15, -20)
		frame.heading:SetPoint("BOTTOMRIGHT", frame.scrollframe, "TOPRIGHT")
		frame.heading.text = frame.heading:CreateFontString("ExplanationHeading", "ARTWORK", "GameFontNormal")
		frame.heading.text:SetText(RawrInputExplanation)
		frame.heading.text:SetAllPoints()
		frame.heading.text:SetJustifyH("LEFT")

		frame.button = CreateFrame('Button', "Import Button", frame, "UIPanelButtonTemplate")
		frame.button:SetText("Import!")
		frame.button:SetWidth(100)
		frame.button:SetHeight(24)
		frame.button:SetPoint("BOTTOM", frame, "BOTTOM", 0, 30)
		frame.button:SetScript("OnClick", function(self, button, up)
			UpgradeList:PerformRawrImport(frame.editbox:GetText())
		end)
		frame.button:Disable()
		
		self.RawrImportFrame = frame
		self.RawrImportFrame:Show()
	end
end

function UpgradeList:PerformRawrImport(text)
	--Credit for a lot of this code goes to the people who wrote the official Rawr addon
	local foundItems = {}
	if string.sub(text, 1, 16) == "Rawr:LoadWebData" then
		text = string.gsub(text, "Rawr:LoadWebData", "UpgradeList:LoadWebData")
	else
		UpgradeList:ChatFrameMessage("Invalid Import!")
	end
	local f, e = loadstring(text)
	if f then
		f()
	else
		UpgradeList:ChatFrameMessage("Problem with import string.  Try recopying from Rawr program.")
		return
	end
	if self.db.ImportData.name ~= UnitName("player") or self.db.ImportData.realm ~= GetRealmName() then
		UpgradeList:ChatFrameMessage("Imported player does not match currently logged in player!")
		return
	end
	if self.db.ImportData.version < "57012" then
		UpgradeList:ChatFrameMessage("Rawr version is too low to import data!")
		return
	end
	foundItems = self.db.ImportData.upgrades
	local counter = 0
	UpgradeList:ChatFrameMessage("Importing from Rawr...")
	for _, item in ipairs(foundItems) do
		if item.item ~= nil then
			GetItemInfo(item.item)
			UpgradeList:ScheduleTimer("ReloadItem", 1, item.item)
			counter = counter + 1
			if counter == #foundItems then
				UpgradeList:ScheduleTimer("CheckJournal", 2, foundItems)
			end			
		end
	end
	
end

function UpgradeList:LoadWebData(data)
	self.db.ImportData = data
end

function UpgradeList:ReloadItem(id)
	GetItemInfo(id)
end

function UpgradeList:CheckJournal(itemsTable) --does NOT actually check the encounter journal anymore
	for _, item in ipairs(itemsTable) do
		local itemName, itemLink, _, itemLevel, _, itemType, itemSubType, _, itemEquipLoc = GetItemInfo(item.item) 
		if itemName ~= nil then
			local _, _, _, _, itemID = string.find(itemLink, "|?c?f?f?(%x*)|?H?([^:]*):?(%d+):?(%d*):?(%d*):?(%d*):?(%d*):?(%d*):?(%-?%d*):?(%-?%d*):?(%d*):?(%d*)|?h?%[?([^%[%]]*)%]?|?h?|?r?")
			local dropLocation = item.loc
			if string.find(dropLocation, "Reward from .+") then --quest item
				self.db.SelectedBoss = ""
				self.db.SelectedInstance = L["Quest Reward"]
				self.db.SelectedDifficulty = ""
			elseif string.find(dropLocation, 'Purchasable with %d+ %[Valor Points%]') or string.find(dropLocation, 'Purchasable with %d+ %[Justice Points%]') or string.find(dropLocation, 'Purchasable with %d+ %[Conquest Points%]') or string.find(dropLocation, 'Purchasable with %d+ %[Honor Points%]') then --can buy the item with points
				_, _, self.db.SelectedBoss, self.db.SelectedInstance = string.find(dropLocation, 'Purchasable with (%d+) %[(.-)%]')
				self.db.SelectedDifficulty = ""
			elseif string.find(dropLocation, "Drops from .+") then --item from a boss
				if string.find(dropLocation, " and Found in ") then --if item is also found in something, cut that part off
					dropLocation = string.gsub(dropLocation, " and Found in .+$", "")
				end
				if string.find(dropLocation, "^%s*Drops%s*from%s*(.-)%s*in%s+(H?e?r?o?i?c?)%s?(%D+)%s%(.+$") then --drop location has odd text (percentage drop, multiple locations listed, etc.)
					_, _, self.db.SelectedBoss,  self.db.SelectedDifficulty, self.db.SelectedInstance = string.find(dropLocation, "^%s*Drops%s*from%s*(.-)%s*in%s+(H?e?r?o?i?c?)%s?(%D+)%s%(.+$")
				else --simple drop location
					_, _, self.db.SelectedBoss,  self.db.SelectedDifficulty, self.db.SelectedInstance = string.find(dropLocation, "%s*Drops%s*from%s*(.-)%s*in%s+(H?e?r?o?i?c?)%s?(.+)$")
				end
				if self.db.SelectedDifficulty == "" then --not heroic difficulty, so it's normal
					self.db.SelectedDifficulty = L["Normal"]
				end
				if string.find(self.db.SelectedInstance, " and Drops from ") then
					self.db.SelectedInstance = string.gsub(self.db.SelectedInstance, " and Drops from .+$", "")
				end
			elseif string.find(dropLocation, "Found in .+") then --items in a container from a boss drop
				_, _, self.db.SelectedBoss,  self.db.SelectedDifficulty, self.db.SelectedInstance = string.find(dropLocation, "%s*Found%s*in%s*(.-)%s*in%s+(H?e?r?o?i?c?)%s?(.+)$")
				if self.db.SelectedDifficulty == "" then
					self.db.SelectedDifficulty = L["Normal"]
				end
			elseif string.find(dropLocation, " crafted ") then --crafted items
				self.db.SelectedBoss = ""
				_, _, self.db.SelectedInstance = string.find(dropLocation, ' crafted (%a+)')
				self.db.SelectedDifficulty = ""
			elseif string.find(dropLocation, " requires ") then --reputation
				_, _, self.db.SelectedInstance, self.db.SelectedBoss = string.find(dropLocation, ' requires (.+) %- (%a+)$')
				self.db.SelectedDifficulty = ""
			elseif string.find(dropLocation, "Trash drop") then --trash drop
				self.db.SelectedBoss = "(Trash drop)"
				_, _, self.db.SelectedDifficulty, self.db.SelectedInstance = string.find(dropLocation, 'Trash drop in (H?e?r?o?i?c?)%s?(.+)$')
				if self.db.SelectedDifficulty == "" then
					self.db.SelectedDifficulty = "Normal"
				end
			elseif string.find(dropLocation, "Purchasable for 1") then --PvP Gear upgrade
				self.db.SelectedBoss = "PvP"
				self.db.SelectedInstance = "(Normal Item Version)"
				self.db.SelectedDifficulty = ""
			elseif string.find(dropLocation, "World Drop") then --World Drop
				self.db.SelectedBoss = ""
				self.db.SelectedInstance = L["World Drop"]
				self.db.SelectedDifficulty = ""
			elseif string.find(dropLocation, "Purchasable with") then --gear from vendors
				self.db.SelectedBoss = "(Purchasable)"
				_, _, self.db.SelectedInstance = string.find(dropLocation, "Purchasable with %d*g%d*s%d*c from %D- in (.+)$")
				self.db.SelectedDifficulty = ""
			elseif string.find(dropLocation, "Sold by") then --catches reputation items a player can buy, but doesn't list what reputation level the player needs
				_, _, self.db.SelectedBoss, self.db.SelectedInstance = string.find(dropLocation, '(Sold by %D-) in (.+)$')
				self.db.SelectedDifficulty = ""
			else
				UpgradeList:ChatFrameMessage("No location match for " .. itemLink .. ".  Item will be added with a location of \"Unknown\".")
				self.db.SelectedBoss = "(Rawr Import)"
				self.db.SelectedInstance = "Unknown"
				self.db.SelectedDifficulty = ""
			end
			local _, englishClass = UnitClass("player")
			if UpgradeList:ItemCanBeWorn(itemSubType, englishClass) then
				UpgradeList:ImportItemsIntoDatabase(self.RawrImportFrame.PrimarySpecRadioButton, itemLink, itemLevel, itemType, itemSubType, itemEquipLoc, itemID)
			end		
		end
	end
	UpgradeList:ChatFrameMessage("Import Complete!")
	UpgradeList.RawrImportFrame:Hide()
end

function UpgradeList:ImportItemsIntoDatabase(PrimarySpecRadioButton, itemLink, itemLevel, itemType, itemSubType, itemEquipLoc, itemID)
	local AddToSpec
	local AddToSpecLong
	if PrimarySpecRadioButton:GetChecked() then
		AddToSpec = L["Primary"]
		AddToSpecLong = "PrimarySpecList"
	else
		AddToSpec = L["Secondary"]
		AddToSpecLong = "SecondarySpecList"
	end
	if itemEquipLoc ~= "" and itemEquipLoc ~= "INVTYPE_BAG" and itemEquipLoc ~= "INVTYPE_TABARD" and itemEquipLoc ~= "INVTYPE_BODY" then
		local _, englishClass = UnitClass("player")
		if englishClass == "WARRIOR" and itemEquipLoc == "INVTYPE_2HWEAPON" then --Adding 2-H weapons to warrior's off-hand list
			self.db.char[AddToSpecLong][self.db.Constants.EquipLocationNumbers[16][3]].Row[self.db.char[AddToSpecLong][self.db.Constants.EquipLocationNumbers[16][3]].Count] = {self.db.char[AddToSpecLong][self.db.Constants.EquipLocationNumbers[16][3]].Count, itemLevel, itemLink, self.db.SelectedDifficulty, self.db.SelectedInstance, self.db.SelectedBoss, "", itemID }
			self.db.char[AddToSpecLong][self.db.Constants.EquipLocationNumbers[16][3]].Count = self.db.char[AddToSpecLong][self.db.Constants.EquipLocationNumbers[16][3]].Count + 1
		end
		for i = 1, #self.db.Constants.EquipLocationNumbers, 1 do
			if itemEquipLoc == self.db.Constants.EquipLocationNumbers[i][1] then
				self.db.char[AddToSpecLong][self.db.Constants.EquipLocationNumbers[i][3]].Row[self.db.char[AddToSpecLong][self.db.Constants.EquipLocationNumbers[i][3]].Count] = {self.db.char[AddToSpecLong][self.db.Constants.EquipLocationNumbers[i][3]].Count, itemLevel, itemLink, self.db.SelectedDifficulty, self.db.SelectedInstance, self.db.SelectedBoss, "", itemID }
				self.db.char[AddToSpecLong][self.db.Constants.EquipLocationNumbers[i][3]].Count = self.db.char[AddToSpecLong][self.db.Constants.EquipLocationNumbers[i][3]].Count + 1
				self.db.SelectedInstance = nil
				self.db.SelectedBoss = nil
				self.db.SelectedDifficulty = nil
			end
		end
		if itemEquipLoc == "INVTYPE_WEAPON" then --Adding 1-H weapons to both Main Hand & Off Hand lists
			self.db.char[AddToSpecLong][self.db.Constants.EquipLocationNumbers[14][3]].Row[self.db.char[AddToSpecLong][self.db.Constants.EquipLocationNumbers[14][3]].Count] = {self.db.char[AddToSpecLong][self.db.Constants.EquipLocationNumbers[14][3]].Count, itemLevel, itemLink, self.db.SelectedDifficulty, self.db.SelectedInstance, self.db.SelectedBoss, "", itemID }
			self.db.char[AddToSpecLong][self.db.Constants.EquipLocationNumbers[14][3]].Count = self.db.char[AddToSpecLong][self.db.Constants.EquipLocationNumbers[14][3]].Count + 1
			
			self.db.char[AddToSpecLong][self.db.Constants.EquipLocationNumbers[16][3]].Row[self.db.char[AddToSpecLong][self.db.Constants.EquipLocationNumbers[16][3]].Count] = {self.db.char[AddToSpecLong][self.db.Constants.EquipLocationNumbers[16][3]].Count, itemLevel, itemLink, self.db.SelectedDifficulty, self.db.SelectedInstance, self.db.SelectedBoss, "", itemID }
			self.db.char[AddToSpecLong][self.db.Constants.EquipLocationNumbers[16][3]].Count = self.db.char[AddToSpecLong][self.db.Constants.EquipLocationNumbers[16][3]].Count + 1
			
			self.db.SelectedInstance = nil
			self.db.SelectedBoss = nil
			self.db.SelectedDifficulty = nil
		end
	else
		self.db.char[AddToSpecLong].Other.Row[self.db.char[AddToSpecLong].Other.Count] = { self.db.char[AddToSpecLong].Other.Count, itemLevel, itemLink, self.db.SelectedDifficulty, self.db.SelectedInstance, self.db.SelectedBoss, "", itemID }
		self.db.char[AddToSpecLong].Other.Count = self.db.char[AddToSpecLong].Other.Count + 1
		self.db.SelectedInstance = nil
		self.db.SelectedBoss = nil
		self.db.SelectedDifficulty = nil
	end
end